/*
 * GUI actions module
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdlib.h>
#include <unistd.h>
#if defined(HAVE_ERRNO_H)
#include <errno.h>
#else defined(HAVE_SYS_ERRNO_H)
#include <sys/errno.h>
#endif
#include <gtk/gtk.h>
#include "gdiff.h"
#include "gui.h"
#include "misc.h"
#include "actions.h"


/**
 * act_mode_change:
 * Change the mode(onepane or twopane) of the view.
 * If the mode of the view is different, look for the same mode view.
 * If it exists, focus it.
 * If it doesn't exist, opne a new view in the current mode.
 */
void
act_mode_change(GDiffFileViews *gfviews, PaneMode pmode)
{
	DiffFiles *dfiles = gfviews->dfiles;
	GDiffDirViews *gdviews = gfviews->gdirviews;
		
	if (gfviews->state.pane_mode != pmode) {
		GSList *list;
		/* Look for opened views which is in current mode. */
		for (list = gdviews->gdfile_list; list; list = list->next) {
			GDiffFileViews *gfviews2;
			
			gfviews2 = (GDiffFileViews*)list->data;
			if (gfviews2->dfiles == dfiles
				&& gfviews2->state.pane_mode == pmode) {
				gint page;
				
				g_assert(gfviews2 != gfviews);
				page = gtk_notebook_page_num(gfviews2->gdwin->notebook, gfviews2->base);
				if (!GTK_WIDGET_VISIBLE(gfviews2->base)) {
					gtk_widget_show(gfviews2->base);
				}
				gtk_notebook_set_page(gfviews2->gdwin->notebook, page);
				return;
			}
		}
		/* When not opened, make it open */	
		gfviews = gdiff_fileviews_new(gdviews->gdwin, gdviews, dfiles, pmode);
		if (pmode == ONE_PANE)
			one_pane_display(gfviews);
		else
			two_pane_display(gfviews);
	}
}

/**
 * act_line_num_show:
 * Toggle show(hide) the line numbers for the view.
 */
void
act_line_num_show(GDiffFileViews *gfviews, gboolean to_show)
{
	if (gfviews->state.pane_mode == ONE_PANE)
		one_pane_show_linenum(gfviews, to_show);
	else
		two_pane_show_linenum(gfviews, to_show);
}


/**
 * act_go_dirview:
 * Look for the appropriate directory view, and focus it.
 */
void
act_go_dirview(GDiffFileViews *gfviews)
{
	const GDiffWindow *gdwin = gfviews->gdwin;
	const GDiffDirViews *gdviews = gfviews->gdirviews;

	if (gdviews->isdir) {/* Comparing two directories */
		gint page;
		page = gtk_notebook_page_num(gdwin->notebook, gdviews->base);
		g_assert(page != -1);
		if (!GTK_WIDGET_VISIBLE(gdviews->base)) {
			g_print("Wrong in go_dirview_cb()\n");
		}
		gtk_notebook_set_page(gdwin->notebook, page);
	}
}

/**
 * act_toggle_text_wrap:
 * Toggle the view's text widget line wrap mode.
 */
void
act_toggle_text_wrap(GDiffFileViews *gfviews, gboolean to_wrap)
{
	if (gfviews->state.pane_mode == ONE_PANE)
		gtk_text_set_line_wrap(gfviews->pane.one.text, to_wrap);
	else {
		int i;
		for (i = 0; i < NUM_COMPARE_FILES; i++) 
			gtk_text_set_line_wrap(gfviews->pane.two.text[i], to_wrap);
	}
}


void
act_edit_file(GDiffFileViews *gfviews, const char *filename)
{
	char *editor;

	editor = getenv("EDITOR");
	if (editor == NULL) {
		g_message("No \"EDITOR\" environment variable\n");
		return;
	} else {
		switch (fork()) {
		case 0:	/* the child */
			close_allfd_with_except(NULL, 0);
			execlp(editor, editor, filename, NULL);
			g_error("\"EDITOR\" environment variable \"%s\" exec error\n", editor);
			gtk_exit(1);
			break;
		case -1:	/* the error */
			g_error("edit_file_cb:fork %d\n", errno);
			break;
		default:/* the parent */
			break;
		}
	}
}
