/* need more improvements */
/*
 * GdiffOverview widget module
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#include "gdiffoverview.h"

/* Constant number */
enum {
  ARG_0,
  ARG_ADJUSTMENT,
};
#define MIN_SLIDER_LENGTH	20

static void gdiff_overview_class_init    (GdiffOverviewClass *klass);
static void gdiff_overview_set_arg (GtkObject          *object,
                    GtkArg             *arg,
                    guint               arg_id);
static void gdiff_overview_get_arg (GtkObject          *object,
                    GtkArg             *arg,
                    guint               arg_id);
static void gdiff_overview_finalize (GtkObject *object);
static void gdiff_overview_init          (GdiffOverview      *overview);
static void gdiff_overview_realize       (GtkWidget           *widget);
static void gdiff_overview_size_allocate (GtkWidget           *widget,
					    GtkAllocation       *allocation);
static void gdiff_overview_send_configure (GdiffOverview     *overview);

static void gdiff_overview_adjustment_changed(GtkAdjustment *adjustment, gpointer data);
//static void gdiff_overview_slider_update (GdiffOverview *overview);
static void gdiff_overview_draw(GtkWidget *widget, GdkRectangle *area);
static gint gdiff_overview_expose (GtkWidget      *widget,
		  GdkEventExpose *event);
//static void gdiff_overview_size_request (GtkWidget      *widget,
//			GtkRequisition *requisition);
static void gdiff_overview_adjustment_value_changed(GtkAdjustment *adjustment, gpointer data);
static void gdiff_overview_draw_slider (GdiffOverview *overview, const GdkRectangle *area);
static void gdiff_overview_clear_background(GtkWidget *widget, const GdkRectangle *area);
static void gdiff_overview_draw_ranges(GdiffOverview *overview, const GdkRectangle *area);
static gint compare_range(const PaintRange *a, const PaintRange *b);


guint
gdiff_overview_get_type (void)
{
  static guint overview_type = 0;

  if (!overview_type) {
      static const GtkTypeInfo overview_info = {
		  "GdiffOverview",
		  sizeof (GdiffOverview),
		  sizeof (GdiffOverviewClass),
		  (GtkClassInitFunc) gdiff_overview_class_init,
		  (GtkObjectInitFunc) gdiff_overview_init,
		  /* reserved_1 */ NULL,
		  /* reserved_2 */ NULL,
		  (GtkClassInitFunc) NULL,
      };
      overview_type = gtk_type_unique (gtk_widget_get_type (), &overview_info);
  }
  
  return overview_type;
}

static void
gdiff_overview_class_init (GdiffOverviewClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

//	g_print("class init\n");
	gtk_object_add_arg_type ("GdiffOverview::adjustment",
							 GTK_TYPE_ADJUSTMENT,
							 GTK_ARG_READWRITE | GTK_ARG_CONSTRUCT,
							 ARG_ADJUSTMENT);
	object_class = (GtkObjectClass*) class;
	widget_class = (GtkWidgetClass*) class;

	object_class->set_arg = gdiff_overview_set_arg;
	object_class->get_arg = gdiff_overview_get_arg;
	object_class->finalize = gdiff_overview_finalize;

	widget_class->realize = gdiff_overview_realize;
	widget_class->size_allocate = gdiff_overview_size_allocate;
//  widget_class->size_request = gdiff_overview_size_request;
	widget_class->expose_event = gdiff_overview_expose;
	widget_class->draw = gdiff_overview_draw;

}

static void
gdiff_overview_set_arg (GtkObject          *object,
					  GtkArg             *arg,
					  guint               arg_id)
{
	GdiffOverview *overview;
  
	overview = GDIFF_OVERVIEW (object);
  
	switch (arg_id) {
	case ARG_ADJUSTMENT:
		gdiff_overview_set_adjustment (GDIFF_OVERVIEW (overview), GTK_VALUE_POINTER (*arg));
		break;
	default:
		break;
	}
}

static void
gdiff_overview_get_arg (GtkObject          *object,
					  GtkArg             *arg,
					  guint               arg_id)
{
	GdiffOverview *overview;
  
	overview = GDIFF_OVERVIEW (object);
  
	switch (arg_id) {
    case ARG_ADJUSTMENT:
		GTK_VALUE_POINTER (*arg) = GDIFF_OVERVIEW (overview);
		break;
    default:
		arg->type = GTK_TYPE_INVALID;
		break;
    }
}

static void
gdiff_overview_finalize (GtkObject *object)
{
	GSList *list;
	GdiffOverview *overview;

	g_print("overview finalize\n");
	overview = GDIFF_OVERVIEW(object);
	for (list = overview->range_list; list; list = list->next) {
		g_free(list->data);
	}
	g_slist_free(overview->range_list);
}

static void
gdiff_overview_init (GdiffOverview *overview)
{
//	g_print("instance init\n");

	overview->adjustment = NULL;
	overview->slider_y = 0;
	overview->slider_length = MIN_SLIDER_LENGTH;
	overview->old_value = 0.0;
	overview->old_lower = 0.0;
	overview->old_upper = 0.0;
	overview->old_page_size = 0.0;
	overview->adjustment = NULL;

	overview->range_list = NULL;
}

GtkWidget*
gdiff_overview_new (GtkAdjustment *adjustment)
{
	GtkWidget *overview;
  
	overview = gtk_widget_new (GTK_TYPE_OVERVIEW,
							   "adjustment", adjustment,
							   NULL);
  
	return overview;
}

void
gdiff_overview_size (GdiffOverview *overview,
				   gint            width,
				   gint            height)
{
	g_return_if_fail (overview != NULL);
	g_return_if_fail (GTK_IS_OVERVIEW (overview));

	GTK_WIDGET (overview)->requisition.width = width;
	GTK_WIDGET (overview)->requisition.height = height;
}

static void
gdiff_overview_realize (GtkWidget *widget)
{
	GdiffOverview *overview;
	GdkWindowAttr attributes;
	gint attributes_mask;

	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_OVERVIEW (widget));

	overview = GDIFF_OVERVIEW (widget);
	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = widget->allocation.width;
	attributes.height = widget->allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.visual = gtk_widget_get_visual (widget);
	attributes.colormap = gtk_widget_get_colormap (widget);
	attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;

	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
	gdk_window_set_user_data (widget->window, overview);

	widget->style = gtk_style_attach (widget->style, widget->window);
	gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);

	/*XXX*/
	gdk_gc_set_function(widget->style->text_gc[GTK_STATE_NORMAL], GDK_XOR);
//  gdiff_overview_slider_update (GDIFF_OVERVIEW (widget)); 
  
//  gdk_window_show (overview->slider);
	gdiff_overview_send_configure (GDIFF_OVERVIEW (widget));//???

}

static void
gdiff_overview_size_allocate (GtkWidget     *widget,
							GtkAllocation *allocation)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_OVERVIEW (widget));
	g_return_if_fail (allocation != NULL);
	
//	g_print("size alloc\n");
	widget->allocation = *allocation;

	if (GTK_WIDGET_REALIZED (widget)) {
		gdk_window_move_resize (widget->window,
								allocation->x, allocation->y,
								allocation->width, allocation->height);

		gdiff_overview_send_configure (GDIFF_OVERVIEW (widget));
    }
}

static void
gdiff_overview_send_configure (GdiffOverview *overview)
{
	GtkWidget *widget;
	GdkEventConfigure event;

	widget = GTK_WIDGET (overview);

	event.type = GDK_CONFIGURE;
	event.window = widget->window;
	event.x = widget->allocation.x;
	event.y = widget->allocation.y;
	event.width = widget->allocation.width;
	event.height = widget->allocation.height;
  
	gtk_widget_event (widget, (GdkEvent*) &event);
}


void
gdiff_overview_set_adjustment (GdiffOverview *overview,
							 GtkAdjustment *adjustment)
{
	g_return_if_fail (overview != NULL);
	g_return_if_fail (GTK_IS_OVERVIEW (overview));
  
	if (!adjustment)
		adjustment = (GtkAdjustment*) gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
	else
		g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));

	if (overview->adjustment != adjustment) {
		if (overview->adjustment) {
			gtk_signal_disconnect_by_data (GTK_OBJECT (overview->adjustment),
										   (gpointer) overview);
			gtk_object_unref (GTK_OBJECT (overview->adjustment));
		}

		overview->adjustment = adjustment;
		gtk_object_ref (GTK_OBJECT (adjustment));
		gtk_object_sink (GTK_OBJECT (adjustment));
      
		gtk_signal_connect (GTK_OBJECT (adjustment), "changed",
							(GtkSignalFunc) gdiff_overview_adjustment_changed,
							(gpointer) overview);
		gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
							(GtkSignalFunc) gdiff_overview_adjustment_value_changed,
							(gpointer) overview);

		overview->old_value = adjustment->value;
		overview->old_lower = adjustment->lower;
		overview->old_upper = adjustment->upper;
		overview->old_page_size = adjustment->page_size;

		gdiff_overview_adjustment_changed (adjustment, (gpointer) overview);
    }
}

static void gdiff_overview_draw(GtkWidget *widget, GdkRectangle *area)
{
//	g_print("draw\n");
}

static gint
gdiff_overview_expose (GtkWidget      *widget,
					 GdkEventExpose *event)
{
	gdiff_overview_clear_background(widget, &event->area);
    gdiff_overview_draw_ranges(GDIFF_OVERVIEW(widget), &event->area);
    gdiff_overview_draw_slider(GDIFF_OVERVIEW(widget), &event->area);

	return FALSE;//XXX
}

#if 0
static void
gdiff_overview_size_request (GtkWidget      *widget,
						   GtkRequisition *requisition)
{
    // refer to gtklabel.c
    g_print("size req\n");
	if (requisition != &widget->requisition)
		*requisition = widget->requisition;
}
#endif

static void
gdiff_overview_adjustment_changed(GtkAdjustment *adjustment, gpointer data)
{
	GdiffOverview *overview;

	g_return_if_fail (adjustment != NULL);
	g_return_if_fail (data != NULL);
	
	overview = GDIFF_OVERVIEW (data);
	
	if (((overview->old_lower != adjustment->lower) ||
		 (overview->old_upper != adjustment->upper) ||
		 (overview->old_page_size != adjustment->page_size)) &&
		(overview->old_value == adjustment->value)) {
		if ((adjustment->lower == adjustment->upper) ||
			(overview->old_lower == (overview->old_upper - overview->old_page_size))) {
			adjustment->value = adjustment->lower;
			gtk_signal_emit_by_name (GTK_OBJECT (adjustment), "value_changed");
		}
    }

	if ((overview->old_value != adjustment->value) ||
		(overview->old_lower != adjustment->lower) ||
		(overview->old_upper != adjustment->upper) ||
		(overview->old_page_size != adjustment->page_size))	{
		overview->old_value = adjustment->value;
		overview->old_lower = adjustment->lower;
		overview->old_upper = adjustment->upper;
		overview->old_page_size = adjustment->page_size;
	}
}

static void
gdiff_overview_adjustment_value_changed(GtkAdjustment *adjustment, gpointer data)
{
	GdiffOverview *overview;

	g_return_if_fail (adjustment != NULL);
	g_return_if_fail (data != NULL);

	overview = GDIFF_OVERVIEW (data);

	if (overview->old_value != adjustment->value) {
//			gdiff_overview_slider_update (overview);
		GdkRectangle rect;
		gint w, h;
		gdk_window_get_size(GTK_WIDGET(overview)->window, &w, &h);
		rect.x = rect.y = 0;
		rect.width = w;
		rect.height = h;

		gdiff_overview_clear_background (GTK_WIDGET(overview), &rect);
		gdiff_overview_draw_ranges(overview, &rect);
		gdiff_overview_draw_slider(overview, &rect);
			
		overview->old_value = adjustment->value;
	}
}

static void
gdiff_overview_draw_slider (GdiffOverview *overview, const GdkRectangle *area)
{
    gint top;
	gint bottom;
	gint y;
	gint width;
	gint height;
	gint length;
	GtkWidget *widget = GTK_WIDGET(overview);
	
	g_return_if_fail (overview != NULL);
	g_return_if_fail (GTK_IS_OVERVIEW (overview));
	g_return_if_fail (GTK_WIDGET(overview)->window != NULL);

	if (overview->adjustment->value < overview->adjustment->lower) {
		overview->adjustment->value = overview->adjustment->lower;
		gtk_signal_emit_by_name (GTK_OBJECT (overview->adjustment), "value_changed");
	}
	else if (overview->adjustment->value > overview->adjustment->upper)	{
		overview->adjustment->value = overview->adjustment->upper;
		gtk_signal_emit_by_name (GTK_OBJECT (overview->adjustment), "value_changed");
	}

	gdk_window_get_size(GTK_WIDGET(overview)->window, &width, &height);
	gdk_draw_rectangle(widget->window, widget->style->text_gc[GTK_STATE_NORMAL],
					   1, 0, y, width, length);
	top = 0;
	bottom = height - overview->slider_length;
	y = top;
	length = overview->slider_length;

	/*XXX: calc slider length */
	if ((overview->adjustment->page_size > 0) &&
		(overview->adjustment->lower != overview->adjustment->upper)) {
		if (overview->adjustment->page_size >
			(overview->adjustment->upper - overview->adjustment->lower))
            overview->adjustment->page_size = overview->adjustment->upper - overview->adjustment->lower;
		
		length = (height * overview->adjustment->page_size /
				  (overview->adjustment->upper - overview->adjustment->lower));
          
		if (length < MIN_SLIDER_LENGTH)
			length = MIN_SLIDER_LENGTH;
	}
      
	/* calc slider pos y */
	if (overview->adjustment->lower != (overview->adjustment->upper - overview->adjustment->page_size))
		y += ((bottom - top) * (overview->adjustment->value - overview->adjustment->lower) /
			  (overview->adjustment->upper - overview->adjustment->lower - overview->adjustment->page_size));
	
	if (y < top)
		y = top;
	else if (y > bottom)
		y = bottom;

	gdk_draw_rectangle(widget->window, widget->style->text_gc[GTK_STATE_NORMAL],
					   1, 0, y, width, length);
	overview->slider_y = y;
	overview->slider_length = length;
}

static void
gdiff_overview_clear_background(GtkWidget *widget, const GdkRectangle *area)
{
	g_return_if_fail (widget->window != NULL);
//	g_print("draw bg\n");
	gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
}


/* Exported interface */
void
gdiff_overview_insert_paintrange(GdiffOverview *overview, gdouble begin, gdouble end)
{
	PaintRange *pr;

	g_return_if_fail (begin >=0 && begin <= 1);
	g_return_if_fail (end >=0 && end <= 1);

	if (begin < 0) begin = 0;
	if (begin > 1) begin = 1;
	if (end < 0) end = 0;
	if (end > 1) end = 1;
	
	pr = g_new(PaintRange, 1);
	pr->begin = begin;
	pr->end = end;
	overview->range_list = g_slist_insert_sorted(overview->range_list, pr, (GCompareFunc)compare_range);
}

static gint
compare_range(const PaintRange *a, const PaintRange *b)
{
	if (a->begin != b->begin)
		return (a->begin > b->begin);
	else
		return (a->end > b->end);
}

static void
gdiff_overview_draw_ranges(GdiffOverview *overview, const GdkRectangle *area)
{
	GSList *node;
	PaintRange *prange;
	GdkRectangle i_rect;//intersect
	GdkRectangle prect;
	gint w, h;


	gdk_window_get_size(GTK_WIDGET(overview)->window, &w, &h);
//	g_print("whole size w=%d h=%d\n", w, h);
	prect.x = 0;
	prect.width = w;
	for (node = overview->range_list; node; node = node->next) {
		prange = (PaintRange*)node->data;
//		g_print("%lf %lf\n", prange->begin, prange->end);
		prect.y = h * prange->begin;
		prect.height = h * (prange->end - prange->begin);
		if (prect.height < 1)
			prect.height = 1;
		if (prect.y == h)
			prect.y--;
		
		if (gdk_rectangle_intersect(area, &prect, &i_rect)) {
//			g_print("range %d %d\n", prect.y, prect.height);		
			gdk_draw_rectangle(GTK_WIDGET(overview)->window,
							   GTK_WIDGET(overview)->style->fg_gc[GTK_STATE_NORMAL],
							   1,
							   prect.x, prect.y,
							   prect.width, prect.height);
		}
	}
}
