/*
 * GUI front-end main header
 * This is the most important file for you to understand front-end.
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#ifndef _GDIFF_DISPLAY_H_
#define _GDIFF_DISPLAY_H_

#include "gdiffoverview.h"

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

/* Forward declarations of types. */
typedef struct _GDiffWindow GDiffWindow;
typedef struct _GDiffDirViews GDiffDirViews;
typedef struct _GDiffFileViews GDiffFileViews;
typedef struct _FontProp FontProp;

/*
 * Contant number
 */
#define WIDTH_OVERVIEW		10/*XXX: hard-code is not good? */
typedef enum {
	ONE_PANE,
	TWO_PANE,
} PaneMode;

/* marks added line numbers. The length should be consistent. */
#define MARK_FILE1		"< "
#define MARK_FILE2		"> "
#define MARK_COMMON		"  "
#define MARK_LENGTH		(sizeof(MARK_COMMON)-1)


/*
 * Data structure definitions
 */

/* Window(equivalent to GtkWindow) data
 * From an intuitive perspective, this is corresponding to one window. */
struct _GDiffWindow {
	GtkWindow *window;
	GtkNotebook *notebook;
	/*GtkWidget *menubar;*/
	/*GtkWidget *toolbar;*/
	GtkFileSelection *filesel;
	
	GList *gdirv_list;/* Doubly linked list of GDiffDirViews */
	GList *gfilev_list;/* Doubly linked list of GDiffFileViews */

	/* flags */
	gboolean f_show_tabs;
};

/* Directory view data
 * From an intuitive perspective, this is corresponding to file list. */
struct _GDiffDirViews {
	/* Back-end data */
	DiffDir *diffdir;
	char *dirname1;
	char *dirname2;
	gboolean isdir;

	GSList *gdfile_list;/* Singly linked list of GDiffFileViews */

	/* For reference */
	GDiffWindow *gdwin;

	/* Front-end data */
	GtkWidget *base;/* A virtual base widget, which points to a real widget. */
	GtkHBox *hbox;
	GtkLabel *label;
	GtkCList *clist;
};

/* File view data
 * From an intuitive perspective, this is corresponding to diff results.
 * Note: Two instances for one back-end data(DiffFiles) can exist,
 * e.g. when a user open it in one-pane mode, and then close it and
 * open it again in two-pane mode.
 * Both instances will be finalized when he closes the directory list.
 */
struct _GDiffFileViews {
	/* Back-end data */
	char *filename1;
	char *filename2;

	/* For reference */
	DiffFiles *dfiles;
	GDiffWindow *gdwin;
	GDiffDirViews *gdirviews;

	/* Front-end data */
	struct {
		PaneMode pane_mode;
		gboolean show_line_num;
	} state;
	GtkWidget *base;/* A virtual base widget, which points to a real widget. */
	union {
		struct {
			GtkHBox *hbox;
			GtkLabel *label;/* For notebook */
			GtkText *text;
			GtkDrawingArea *darea;
			GdiffOverview *overview[NUM_COMPARE_FILES];
			GtkMenu *pmenu;/* popup menu */
		} one;
		struct {
			GtkHBox *hbox;
			GtkLabel *label;/* For notebook */
			GtkText *text[NUM_COMPARE_FILES];
			GtkDrawingArea *darea;
			GdiffOverview *overview[NUM_COMPARE_FILES];
			GtkVScrollbar *vscrollboth;
			GtkMenu *pmenu[NUM_COMPARE_FILES];/* popup menu */
		} two;
	} pane;

	GdkColor *fcolor[NUM_COMPARE_FILES];
	int merged_nline;
};


/* Font property */
struct _FontProp {
	GdkFont *font;
	GdkColor *fg;
	GdkColor *bg;
};

/* Preference
 * XXX: Later, will be extended more, and moved to different header file. */
typedef struct {
	PaneMode pane_mode;
	gboolean show_line_num;
	gboolean line_wrap;
} Preference;


/* Global variable declarations */
Preference g_pref;


/* Global function declarations */
/* files.c */ 
extern void files_open(GDiffWindow *gdwin);

/* menu.c */
GtkWidget* create_menus(GtkWidget *window, GDiffWindow *gdwin);

/* rmenu.c */
extern void create_rightclick_menu(GDiffFileViews *gfileviews);
extern void rightclick_menu_cb(GtkWidget *w, GdkEventButton *event, gpointer data);

/* guiwin.c */
extern GDiffWindow* gdiff_window_new(void);

/* guiviews.c */
extern GDiffDirViews* gdiff_dirviews_new(GDiffWindow *gdwin, const char *fname1, const char *fname2, gboolean isdir);
extern void gdiff_dirviews_delete(GDiffDirViews *gdirviews);
extern GDiffFileViews* gdiff_fileviews_new(GDiffWindow *gdwin, GDiffDirViews *gdirviews, DiffFiles *dfiles, PaneMode pmode);
extern void* gdiff_current_views(const GDiffWindow *gdwin, gboolean *ret_isdir);

/* onepane.c */
extern void create_onepane_widgets(GDiffFileViews *gfileviews);
extern void one_pane_display(GDiffFileViews *gfileviews);
extern void one_pane_show_linenum(GDiffFileViews *gfileviews, gboolean to_show);
/*extern void one_pane_next_prev(GDiffFileViews *gfileviews, gboolean to_next);*/

/* twopane.c */
extern void create_twopane_widgets(GDiffFileViews *gfileviews);
extern void two_pane_display(GDiffFileViews *gfileviews);
extern void two_pane_show_linenum(GDiffFileViews *gfileviews, gboolean to_show);
/*extern void two_pane_next_prev(GDiffFileViews *gfileviews, gboolean to_next);*/

/* dirview.c */
extern void create_dir_widgets(GDiffDirViews *gdirviews);
extern void display_dirs(GDiffDirViews *gdirviews);

/* guimisc.c */
extern int draw_text_lines(GtkText *text, const FontProp *fprop, const char *buf, int nl, int max_lenb);
extern int insert_remove_line_numbers(GtkText *text, gboolean b_in, int *pos, const FontProp *fprop, const char *buf, int begin, int nline, int max_lenb, int n_col, const char *mark);
extern GtkWidget* make_label_for_notebook(const char *f1, const char *f2);

#ifdef __cplusplus
} 
#endif /* __cplusplus */
 
#endif /* __GDIFF_DISPLAY_H__ */
