/*
 * GUI front-end
 * GDiffWindow module
 * See "gui.h" for the details of data structure.
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#include <gtk/gtk.h>
#include "gdiff.h"
#include "gui.h"


/**
 * gdiff_window_new:
 * Allocate GDiffWindow, initialize it, and return its pointer.
 * No related back-end data, so what this function does is almost just
 * creating a window.
 * Now, its returned value is not used.
 * Instead, implicitly pass GDiffWindow to callback-functions of menu.
 * Input:
 * None;
 * Output:
 * Return value; Allocated GDiffWindow.
 */
GDiffWindow*
gdiff_window_new(void)
{
	GDiffWindow *gdiffwin;
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *menu;
	GtkWidget *notebook;

	gdiffwin = g_new(GDiffWindow, 1);

	/* Initialize */
	gdiffwin->filesel = NULL;
	gdiffwin->gdirv_list = NULL;
	gdiffwin->gfilev_list = NULL;
	gdiffwin->f_show_tabs = TRUE;

	/* Main window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gdiffwin->window = GTK_WINDOW(window);
	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
	gtk_widget_set_name(window, "gtkdiff");
	gtk_window_set_title(GTK_WINDOW (window), "gtkdiff");
	gtk_widget_set_usize(window, 500, 400);
	gtk_widget_set_uposition(window, 20, 20);
	
	/* It's a good idea to do this for all windows. */
	/*TODO: for supporting multiple windows, to connect gdiff_window_delete*/
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
						GTK_SIGNAL_FUNC(gtk_exit), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
						GTK_SIGNAL_FUNC(gtk_exit), NULL);

	/* vbox */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	/* menu */
	menu = create_menus(window, gdiffwin);
	gtk_box_pack_start(GTK_BOX(vbox), menu, FALSE, FALSE, 0);
	/* notebook */
	notebook = gtk_notebook_new();
	gdiffwin->notebook = GTK_NOTEBOOK(notebook);
	gdiffwin->f_show_tabs = TRUE;
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), gdiffwin->f_show_tabs);
	gtk_notebook_set_homogeneous_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
	/*???	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);*/
	gtk_notebook_popup_enable(GTK_NOTEBOOK(notebook));
	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	
	gtk_widget_show_all(window);

	return gdiffwin;
}

/*place holder*/
void
gdiff_window_delete(GDiffWindow *gdwin)
{

	return;
}
