/*
 * Module almost for main() function and some initializations.
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <gtk/gtk.h>
#include "gdiff.h"
#include "gui.h"
#include "misc.h"

/* XXX: rc file */
#define GTKDIFFRC	"gtkdiffrc"

/* Private function declarations */
static void rc_file_init(void);
static void gtkdiff_init(void);
static void argument_init(GDiffWindow *gdwin, int argc, char *argv[]);


int
main(int argc, char *argv[])
{
	GDiffWindow *gdiffwin;
	
	/* Initialize */
	gtk_set_locale();
	rc_file_init();
	gtk_init(&argc, &argv);
	gdk_rgb_init();

	gtkdiff_init();
	gdiffwin = gdiff_window_new();
	argument_init(gdiffwin, argc, argv);
	
	gtk_main();
  
	return 0;
}



/* ---The followings are private functions--- */
/*XXX: a quick hack. What is a proper way to take care of rc file? */
/**
 * rc_file_init:
 * Search for rc file in the current directory and home directory.
 */
static void
rc_file_init(void)
{
	char *homedir;
	char path[PATH_MAX];

	/* Current directory */
	if (access(GTKDIFFRC, R_OK) == 0) {
		gtk_rc_add_default_file(GTKDIFFRC);
		return;
	}

	/* Home directory */
	homedir = getenv("HOME");
	if (homedir == NULL)
		return;
	sprintf(path, "%s/%s", homedir, GTKDIFFRC);
	if (access(path, R_OK) == 0) {
		gtk_rc_add_default_file(path);
		return;
	}
	g_warning("rc file(gtkdiffrc) not found.\nIt should be in the current directory or home directory.\n");
}

/**
 * gtkdiff_init:
 * Later, it will do more.
 */
static void
gtkdiff_init(void)
{
	signal(SIGCHLD, def_signal_handler);

	/* Preference */
	g_pref.pane_mode = ONE_PANE;
	g_pref.show_line_num = TRUE;
	g_pref.line_wrap = FALSE;
}


/**
 * argument_init:
 * Just take care of specifying two file names for command option.
 */
static void
argument_init(GDiffWindow *gdwin, int argc, char *argv[])
{
	GDiffDirViews *gdirviews;
	gboolean is_dir;
	FileType ftype1, ftype2;
	
	if (argc != 3)
		return;

	ftype1 = check_filetype(argv[1]);
	ftype2 = check_filetype(argv[2]);
	if (ftype1 != ftype2) {
		g_message("you can't compare file and directory.\n");
		return;
	}
	if (ftype1 == REGULARFILE)
		is_dir = FALSE;
	else if (ftype1 == DIRECTORY)
		is_dir = TRUE;
	else
		return;
	gdirviews = gdiff_dirviews_new(gdwin, argv[1], argv[2], is_dir);
	display_dirs(gdirviews);
}

