/*
 * GUI menu bar module
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#include <gtk/gtk.h>
#include "gdiff.h"
#include "gui.h"
#include "actions.h"


/* This module will be rewritten after porting to Gnome.
 * Rather, now it is a very bad UI... */

/* Private function declarations */
static void file_open_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void file_close_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void mode_change_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void line_num_show_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void go_dirview_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
/*static void move_next_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);*/
static void text_wrap_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gtk_ifactory_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);



/* Private variables */
static GtkItemFactoryEntry menu_items[] =
{
  { "/_File",            NULL,         0,                     0, "<Branch>" },
/*  { "/File/tearoff1",    NULL,         0,       0, "<Tearoff>" },*/
  { "/File/_Open",       "<control>O", file_open_cb,       0 },
  { "/File/_Close",      "<control>W", file_close_cb,       0 },
  { "/File/sep1",        NULL,         gtk_ifactory_cb,       0, "<Separator>" },
  { "/File/_Quit",       "<control>Q", gtk_main_quit,       0 },

  { "/_Preference", NULL, 0, 0, "<Branch>" },
  { "/Preference/Text_Wrap", NULL, 0, 0, "<Branch>" },
  { "/Preference/TextWrap/_Yes", NULL, text_wrap_cb, TRUE, "<RadioItem>" },
  { "/Preference/TextWrap/_No", NULL, text_wrap_cb, FALSE, "/Preference/TextWrap/Yes" },

  { "/_Mode", NULL, 0, 0, "<Branch>" },
  { "/Mode/_OnePane", NULL, mode_change_cb, ONE_PANE, "<RadioItem>" },
  { "/Mode/_TwoPanes", NULL, mode_change_cb, TWO_PANE, "/Mode/OnePane" },

  { "/_LineNumber", NULL,	0, 0, "<Branch>" },
  { "/LineNumber/_Show", NULL,	line_num_show_cb, TRUE, "<RadioItem>" },
  { "/LineNumber/_Hide", NULL, line_num_show_cb, FALSE, "/LineNumber/Show" },

  { "/_Go", NULL,	0, 0 , "<Branch>" },
  { "/Go/_Directory Pane", NULL,	go_dirview_cb, 0 },
  /*
  { "/_Action", NULL,	NULL, 0, "<Branch>" },
  { "/Action/_Next diff", NULL,	move_next_cb, TRUE },
  { "/Action/_Prev diff", NULL,	move_next_cb, FALSE },
  */

  { "/_Help",            NULL,         0,                     0, "<LastBranch>" },
  { "/Help/_About",      NULL,         gtk_ifactory_cb,       0 },
};
static const int NMENU_ITEMS = sizeof(menu_items) / sizeof(menu_items[0]);


/**
 * create_menus:
 */
GtkWidget*
create_menus(GtkWidget *window, GDiffWindow *gdwin)
{
	GtkAccelGroup *accel_group;
	GtkItemFactory *item_factory;
      
	accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);

	gtk_item_factory_create_items(item_factory, NMENU_ITEMS, menu_items, gdwin);
	gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
      
	return gtk_item_factory_get_widget(item_factory, "<main>");
}


/* ---The followings are private functions--- */
/**
 * file_open_cb:
 */
static void
file_open_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;

	files_open(gdwin);
}

/**
 * file_close_cb:
 * XXX: the logic should be moved to actions.c?
 */
static void
file_close_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	gboolean isdir;
	void *p;
	GDiffDirViews *gdviews;

	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir) {
		gdviews = (GDiffDirViews*)p;
		gdiff_dirviews_delete(gdviews);
	} else {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		gdviews = gfviews->gdirviews;
		if (gdviews->isdir)
			gtk_widget_hide(GTK_WIDGET(gfviews->base));			
		else /* Comparing two files */
			gdiff_dirviews_delete(gdviews);
	}		
}

/**
 * mode_change_cb:
 * Change the mode(onepane or twopane), and set a global variable, g_pref.
 */
static void
mode_change_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	PaneMode pmode = (PaneMode)callback_action;
	gboolean isdir;
	void *p;

	g_pref.pane_mode = pmode;
	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir == FALSE) {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		act_mode_change(gfviews, pmode);
	}
}

/**
 * line_num_show_cb:
 * Toggle show(hide) the line numbers for the current view.
 * Also, set a global variable, "g_pref".
 */
static void
line_num_show_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	gboolean isdir;
	void *p;

	g_pref.show_line_num = (gboolean)callback_action;

	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir == FALSE) {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		act_line_num_show(gfviews, g_pref.show_line_num);
	}
}

/**
 * go_dirview_cb:
 * If comparing two directories, and the current view is a file view,
 * look for the appropriate directory view, and focus it.
 */
static void
go_dirview_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	gboolean isdir;
	void *p;

	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir == FALSE) {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		act_go_dirview(gfviews);
	}
}

#if 0
/*XXX not yet*/
static void
move_next_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	gboolean to_next = (gboolean)callback_action;
	gboolean isdir;
	void *p;

	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir == FALSE) {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		if (gfviews->state.pane_mode == ONE_PANE)
			one_pane_next_prev(gfviews, to_next);
		else
			two_pane_next_prev(gfviews, to_next);
	}
}
#endif

static void text_wrap_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	GDiffWindow *gdwin = (GDiffWindow*)callback_data;
	gboolean isdir;
	void *p;

	g_pref.line_wrap = (gboolean)callback_action;
	p = gdiff_current_views(gdwin, &isdir);
	if (p == NULL) {
		return;
	}
	if (isdir == FALSE) {
		GDiffFileViews *gfviews = (GDiffFileViews*)p;
		act_toggle_text_wrap(gfviews, g_pref.line_wrap);
	}
}

/*place holder*/
static void
gtk_ifactory_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	g_message("ItemFactory: activated \"%s\"", gtk_item_factory_path_from_widget (widget));
}
/* --- Menu End ---*/

