/*
 * GUI right click menu module
 *
 * Programmed by INOUE Seiichiro <inoue@ainet.or.jp>
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <gtk/gtk.h>
#include "gdiff.h"
#include "gui.h"
#include "actions.h"


/* Private function declarations */
static void mode_change_cb(GtkWidget *w, gpointer data);
static void line_num_show_cb(GtkWidget *w, gpointer data);
static void go_dirview_cb(GtkWidget *w, gpointer data);
static void text_wrap_cb(GtkWidget *w, gpointer data);
static void edit_file_cb(GtkWidget *w, gpointer data);


/* Private data structure definitions */
struct RMenuEntry {
	char *label;
	GtkSignalFunc callback;
};

void
create_rightclick_menu(GDiffFileViews *gfileviews)
{
	static struct RMenuEntry menu_items[] = {
		{"pane mode", mode_change_cb},
		{"show(hide) line numbers", line_num_show_cb},
		{"go directory", go_dirview_cb},
		{"toggle line wrap", text_wrap_cb},
		{"edit file", edit_file_cb},
	};
	static const int NMENU_ITEMS = sizeof(menu_items) / sizeof(menu_items[0]);
	PaneMode pmode = gfileviews->state.pane_mode;
	GtkWidget *menu;

	if (pmode == ONE_PANE) {
		int n;
		menu = gtk_menu_new();
		for (n = 0; n < NMENU_ITEMS; n++) {
			GtkWidget *menuitem;
			menuitem = gtk_menu_item_new_with_label(menu_items[n].label);
			gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
							   menu_items[n].callback, gfileviews);
			gtk_menu_append(GTK_MENU(menu), menuitem);
			gtk_widget_show(menuitem);
		}
		gfileviews->pane.one.pmenu = GTK_MENU(menu);
	} else {
		int i;
		for (i = 0; i < NUM_COMPARE_FILES; i++) {
			int n;
			menu = gtk_menu_new();
			for (n = 0; n < NMENU_ITEMS; n++) {
				GtkWidget *menuitem;
				menuitem = gtk_menu_item_new_with_label(menu_items[n].label);
				gtk_object_set_user_data(GTK_OBJECT(menuitem), (gpointer)i);
				gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
								   menu_items[n].callback, gfileviews);
				gtk_menu_append(GTK_MENU(menu), menuitem);
				gtk_widget_show(menuitem);
			}
			gfileviews->pane.two.pmenu[i] = GTK_MENU(menu);
		}
	}
}

void
rightclick_menu_cb(GtkWidget *w, GdkEventButton *event, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;
	PaneMode pmode = gfileviews->state.pane_mode;
	GtkMenu *menu;

	if (pmode == ONE_PANE) {
		menu = gfileviews->pane.one.pmenu;
	} else {
		WhichFile n = (WhichFile)gtk_object_get_user_data(GTK_OBJECT(w));
		menu = gfileviews->pane.two.pmenu[n];
	}	
	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
		gtk_menu_popup(menu, NULL, NULL, (GtkMenuPositionFunc)NULL, NULL,
					   event->button, event->time);
	}
}


/* ---The followings are private functions--- */
static void
mode_change_cb(GtkWidget *w, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;

	act_mode_change(gfileviews, !gfileviews->state.pane_mode);
}

static void
line_num_show_cb(GtkWidget *w, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;

	act_line_num_show(gfileviews, !gfileviews->state.show_line_num);
}

static void
go_dirview_cb(GtkWidget *w, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;

	act_go_dirview(gfileviews);
}

static void
text_wrap_cb(GtkWidget *w, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;
	WhichFile n = (WhichFile)gtk_object_get_user_data(GTK_OBJECT(w));
	GtkText *text = gfileviews->pane.two.text[n];
	gboolean to_wrap;

	to_wrap = !text->line_wrap;
	act_toggle_text_wrap(gfileviews, to_wrap);
}

/*experimental*/
static void
edit_file_cb(GtkWidget *w, gpointer data)
{
	GDiffFileViews *gfileviews = (GDiffFileViews*)data;
	DiffFiles *dfiles = gfileviews->dfiles;
	PaneMode pmode = gfileviews->state.pane_mode;
	WhichFile n;
	const FileInfo *fi;
	char *filename; 

	if (pmode == ONE_PANE)
		n = FIRST_FILE;
	else
		n = (WhichFile)gtk_object_get_user_data(GTK_OBJECT(w));
	fi = get_fileinfo_from_diff_files(dfiles, n, FALSE);
	filename = fi->fname;

	act_edit_file(gfileviews, filename);
}

